'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.task = task;

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _openurl = require('openurl');

var _openurl2 = _interopRequireDefault(_openurl);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var path = require('path');
var _ = require('lodash');
var gutil = require('gulp-util');
var through = require('through2');

var debug = require('debug')('tasks.babel');


var didLastRunFail = false;

function getOpts(opts) {

  return _.defaults(opts, {
    srcGlob: '',
    srcFilesGlob: 'src/**/!(*.js|*.jsx)',
    shouldCompileTests: false,
    destDir: '',
    testsGlob: '',
    babel: {} });
}

// See src/config.
function task(gulp, opts) {

  opts = getOpts(opts);

  gulp.task('lib:compile', gulp.series(['lib:compile-js', 'lib:copy-files']));

  gulp.task('lib:compile-js', function () {
    var gulpif = require('gulp-if');
    var changed = require('gulp-changed');
    var ignore = require('gulp-ignore');
    var babel = require('gulp-babel');
    var sourcemaps = require('gulp-sourcemaps');
    var cwd = require('cwd');

    return gulp.src(opts.srcGlob).pipe(gulpif(gulp.isWatching, changed(opts.destDir))).pipe(gulpif(!opts.shouldCompileTests, ignore.exclude(opts.testsGlob))).pipe(sourcemaps.init()).pipe(babel(opts.babel)).on('error', onError)
    // Source maps work with IntelliJ debugger using Node.js 0.12.7.
    .pipe(sourcemaps.write('.', {
      includeContent: false,
      sourceRoot: function sourceRoot(file) {
        // TODO(vjpr): This might be an issue in the browser? Need to test the change.
        //return path.join(path.relative(file.path, cwd()), opts.srcDir)
        return path.join(path.relative(path.dirname(file.path), cwd()), opts.srcDir);
      }
    })).pipe(gulp.dest(opts.destDir)).pipe(logFileHelpers()).on('end', function (e) {
      // TODO(vjpr)
      //if (didLastRunFail) {
      //  console.log('fixed!')
      //}
      //didLastRunFail = false
    });
  });

  gulp.task('lib:copy-files', function () {
    return gulp.src(opts.srcFilesGlob).pipe(gulp.dest(opts.destDir));
  });

  gulp.task('lib:clean', function () {
    var del = require('del');
    var gutil = require('gulp-util');

    gutil.log('Cleaning:', gutil.colors.yellow(opts.destDir));
    return del(opts.destGlob);
  });

  gulp.task('lib:watch', gulp.series('lib:clean', 'lib:compile', function watcher() {
    var gutil = require('gulp-util');

    // Used by `compile` task to check whether we should only compile changed files.
    gulp.isWatching = true;

    gulp.watch(opts.srcGlob, gulp.series('lib:compile')).on('change', function (evt) {
      gutil.log('---');
      gutil.log('File changed:', gutil.colors.yellow(evt));
    });
  }));
}

////////////////////////////////////////////////////////////////////////////////

// Babel logging

//const babelCache = {}

function logFileHelpers() {

  return through.obj(function (file, enc, cb) {

    // Log

    if (file.extname === '.map') return cb(null, file);

    var src = file.history[0];
    var dest = file.history[1];

    //const relPath = file.relative // NOTE: Doesn't include lib.
    var relPath = function relPath(f) {
      return path.relative('.', f);
    };

    //gutil.log('Compiling (Babel):', gutil.colors.red(relPath(src) + ' -> ' + relPath(dest)))
    gutil.log(_chalk2.default.green(relPath(dest)));

    // Collect babel information

    //gutil.log(file.babel.usedHelpers)
    //gutil.log(file.babel && JSON.stringify(file.babel.modules, null, 2))
    //babelCache[file] = file.babel

    cb(null, file);
  });
}

////////////////////////////////////////////////////////////////////////////////

function notify(e) {
  var message = e.message;
  var fileName = e.fileName;
  var name = e.name;
  var loc = e.loc;

  var notifier = require('node-notifier');
  var relPath = path.relative(process.cwd(), fileName);
  var line = loc && loc.line;
  message = name + ': ' + message.replace(fileName + ': ', '');
  var url = 'idea://open?file=' + fileName + '&line=' + line;
  //const url = 'http://google.com'
  var projectName = process.cwd().split('/').slice(-1);
  var title = projectName;
  var subtitle = relPath + ':' + line;
  notifier.on('click', function (notifierObject, options) {
    debug('Opening url:', url);
    _openurl2.default.open(url);
  });
  notifier.notify({
    title: title,
    message: message,
    subtitle: subtitle,
    icon: path.resolve(__dirname, '../../images/babel.png'),
    // TODO(vjpr): On fixed error use tick icon.
    //contentImage: path.resolve(__dirname, '../../images/tick.png'),
    contentImage: null,
    //open: url, // TODO(vjpr): Not working.
    wait: true });
}

// TODO(vjpr): Not working.
function onError(e) {
  didLastRunFail = true;
  notify(e);
  gutil.log('Error', _chalk2.default.red(parseError(e)));
  this.emit('end');
}

// This is what a standard error looks like:
//
//    /Users/Vaughan/dev-live/firmware-simulator/src/modules/ranging-messages.js: Unexpected token (7:2)
//
// We want to combine the file name and line number for more precision hyperlinking
// when using iTerm2.
function parseError(e) {
  var lineNumber = e.loc ? ':' + e.loc.line + ':' + e.loc.column : '';
  var fileLink = '' + e.fileName + lineNumber;
  // Replace file name.
  var msg = e.message.replace(e.fileName, fileLink);
  return msg + '\n' + e.codeFrame;
}