import path from 'path'
import _ from 'lodash'
import chalk from 'chalk'
import config from './config'
import fs from 'fs'

module.exports = main

////////////////////////////////////////////////////////////////////////////////

const tasksDir = './tasks'

// opts - Allows config to be passed in from the gulpfile. However, it is better
//   to use the babelator.js config file because it allows us to not rely on
//   gulp in the future.
function main(gulp, opts = {}) {

  const FwdRef = require('undertaker-forward-reference')

  // This allows forward references to tasks.
  const fwdReferenceRegistry = new FwdRef
  const registry = fwdReferenceRegistry
  gulp.registry(fwdReferenceRegistry)

  require('gulp-help')(gulp, {
    hideDefaultTask: true,
    hideEmpty: true, // Doesn't exist yet...https://github.com/chmontgomery/gulp-help/issues/36
  })

  //////////////////////////////////////////////////////////////////////////////

  // TODO(vjpr): Extract common `src/dist` paths.
  opts = config(opts)

  // TODO(vjpr): Maybe make this a setter and enforce who can change it and when for better safety.
  // Globally stores the config profile to use when running tasks. Tasks using the `getOpts` function will receive `opts` for this particular profile.
  gulp.profile = null

  // This function is the 3rd argument passed to each task module. This function *should* be called inside each task, allowing task-specific configuration.
  function getOpts() {
    if (!gulp.profile) return opts
    if (!opts[gulp.profile]) {
      console.error('A profile was activated (by setting `gulp.profile`) but the profile configuration is missing (from `babelator.js`).')
      process.exit(1)
      return
    }
    return opts[gulp.profile]
  }

  //////////////////////////////////////////////////////////////////////////////

  // Require all modules in `./tasks` dir, and pass `gulp` and `opts` to each module.

  const fullDir = path.join(__dirname, tasksDir)
  const modules = requireDir(fullDir)
  _(modules).forEach((module) => {
    module.task && module.task(gulp, opts, getOpts)
  })

  gulp.task('default', gulp.series('help'))
  gulp.task('compile', d(`Compile ${chalk.bold(opts.srcDir)} to ${chalk.bold(opts.destDir)}`, gulp.series('lib:compile')))
  gulp.task('test', d('Compile and run tests on transpiled tests', gulp.series('mocha:lib')))
  gulp.task('test-src', d(`Run tests using glob: ${chalk.bold(opts.testsGlob)}`, gulp.series('mocha:src')))
  gulp.task('watch', d('Compile on file change', gulp.series('lib:watch')))
  gulp.task('test-watch', d('Compile and run tests on file change', gulp.series('test:watch')))
  gulp.task('build', d('Clean and compile', gulp.series('lib:clean', 'lib:compile')))
  gulp.task('show-config', d('Show resolved config settings', (done) => {
    console.log(`\n${chalk.underline('Resolved config')}\n`)
    pp(opts)
    console.log('')
    const p = config.getConfigPath()
    if (fs.existsSync(p)) {
      console.log('Using config from:', chalk.bold(p))
      console.log('All possible config options can be found by looking in:', chalk.bold(tasksDir))
    } else {
      console.log('To modify config created file:', p)
    }
    console.log('')
    done()
  }))

  // Run all tasks ending with `:bootstrap`.
  // TODO(vjpr): Deprecated. We now use Yeoman Generators.
  const bootstrapTasks = _(registry._tasks).filter((v, k) => _.endsWith(k, ':bootstrap')).value()
  gulp.task('bootstrap', gulp.series(_.values(bootstrapTasks)))
  gulp.task('prepublish', gulp.series('npm:prepublish'))

}

////////////////////////////////////////////////////////////////////////////////
// Utilities
////////////////////////////////////////////////////////////////////////////////

function pp(...args) {
  const prettyjson = require('prettyjson')

  console.log(prettyjson.render(...args))
}

function requireDir(fullDir) {
  return require('fs').readdirSync(fullDir).map((entry) => {
    try {
      return require(path.join(fullDir, entry))
    } catch (e) {
      if (e.name !== 'MODULE_NOT_FOUND') {
        throw e
      }
    }
  })
}

function d(description, fn) {
  fn.description = description
  return fn
}

////////////////////////////////////////////////////////////////////////////////
