const path = require('path')
const _ = require('lodash')
const gutil = require('gulp-util')
const through = require('through2')
import chalk from 'chalk'
const debug = require('debug')('tasks.babel')
import openurl from 'openurl'

let didLastRunFail = false

function getOpts(opts) {

  return _.defaults(opts, {
    srcGlob: '',
    srcFilesGlob: 'src/**/!(*.js|*.jsx)',
    shouldCompileTests: false,
    destDir: '',
    testsGlob: '',
    babel: {}, // See src/config.
  })

}

export function task(gulp, opts) {

  opts = getOpts(opts)

  gulp.task('lib:compile', gulp.series(['lib:compile-js', 'lib:copy-files']))

  gulp.task('lib:compile-js', () => {
    const gulpif = require('gulp-if')
    const changed = require('gulp-changed')
    const ignore = require('gulp-ignore')
    const babel = require('gulp-babel')
    const sourcemaps = require('gulp-sourcemaps')
    const cwd = require('cwd')

    return gulp.src(opts.srcGlob)
      .pipe(gulpif(gulp.isWatching, changed(opts.destDir)))
      .pipe(gulpif(!opts.shouldCompileTests, ignore.exclude(opts.testsGlob)))
      .pipe(sourcemaps.init())
      .pipe(babel(opts.babel))
      .on('error', onError)
      // Source maps work with IntelliJ debugger using Node.js 0.12.7.
      .pipe(sourcemaps.write('.', {
        includeContent: false,
        sourceRoot: file => {
          // TODO(vjpr): This might be an issue in the browser? Need to test the change.
          //return path.join(path.relative(file.path, cwd()), opts.srcDir)
          return path.join(path.relative(path.dirname(file.path), cwd()), opts.srcDir)
        }
      }))
      .pipe(gulp.dest(opts.destDir))
      .pipe(logFileHelpers())
      .on('end', (e) => {
        // TODO(vjpr)
        //if (didLastRunFail) {
        //  console.log('fixed!')
        //}
        //didLastRunFail = false
      })
  })

  gulp.task('lib:copy-files', () => {
    return gulp.src(opts.srcFilesGlob).pipe(gulp.dest(opts.destDir))
  })

  gulp.task('lib:clean', () => {
    const del = require('del')
    const gutil = require('gulp-util')

    gutil.log('Cleaning:', gutil.colors.yellow(opts.destDir))
    return del(opts.destGlob)
  })

  gulp.task('lib:watch', gulp.series('lib:clean', 'lib:compile', function watcher() {
    const gutil = require('gulp-util')

    // Used by `compile` task to check whether we should only compile changed files.
    gulp.isWatching = true

    gulp.watch(opts.srcGlob, gulp.series('lib:compile')).on('change', (evt) => {
      gutil.log('---')
      gutil.log('File changed:', gutil.colors.yellow(evt))
    })

  }))


}

////////////////////////////////////////////////////////////////////////////////

// Babel logging

//const babelCache = {}

function logFileHelpers() {

  return through.obj((file, enc, cb) => {

    // Log

    if (file.extname === '.map') return cb(null, file)

    var src = file.history[0]
    var dest = file.history[1]

    //const relPath = file.relative // NOTE: Doesn't include lib.
    const relPath = f => path.relative('.', f)

    //gutil.log('Compiling (Babel):', gutil.colors.red(relPath(src) + ' -> ' + relPath(dest)))
    gutil.log(chalk.green(relPath(dest)))

    // Collect babel information

    //gutil.log(file.babel.usedHelpers)
    //gutil.log(file.babel && JSON.stringify(file.babel.modules, null, 2))
    //babelCache[file] = file.babel

    cb(null, file)

  })
}

////////////////////////////////////////////////////////////////////////////////


function notify(e) {
  let {message, fileName, name, loc} = e
  const notifier = require('node-notifier')
  const relPath = path.relative(process.cwd(), fileName)
  const line = loc && loc.line
  message = name + ': ' + message.replace(fileName + ': ', '')
  const url = `idea://open?file=${fileName}&line=${line}`
  //const url = 'http://google.com'
  const projectName = process.cwd().split('/').slice(-1)
  const title = projectName
  const subtitle = relPath + ':' + line
  notifier.on('click', (notifierObject, options) => {
    debug('Opening url:', url)
    openurl.open(url)
  })
  notifier.notify({
    title,
    message,
    subtitle,
    icon: path.resolve(__dirname, '../../images/babel.png'),
    // TODO(vjpr): On fixed error use tick icon.
    //contentImage: path.resolve(__dirname, '../../images/tick.png'),
    contentImage: null,
    //open: url, // TODO(vjpr): Not working.
    wait: true, // TODO(vjpr): Not working.
  })
}

function onError(e) {
  didLastRunFail = true
  notify(e)
  gutil.log('Error', chalk.red(parseError(e)))
  this.emit('end')
}

// This is what a standard error looks like:
//
//    /Users/Vaughan/dev-live/firmware-simulator/src/modules/ranging-messages.js: Unexpected token (7:2)
//
// We want to combine the file name and line number for more precision hyperlinking
// when using iTerm2.
function parseError(e) {
  const lineNumber = e.loc ? `:${e.loc.line}:${e.loc.column}` : ``
  const fileLink = `${e.fileName}` + lineNumber
  // Replace file name.
  const msg = e.message.replace(e.fileName, fileLink)
  return msg + '\n' + e.codeFrame
}
